@@ -22,13 +22,12 @@ use crate::{
     conversion::{
         analysis::{
             fun::function_wrapper::CppFunctionKind,
-            has_attr,
-            type_converter::{add_analysis, TypeConversionContext, TypeConverter},
+            type_converter::{self, add_analysis, TypeConversionContext, TypeConverter},
         },
         api::{ApiName, FuncToConvert, SubclassName},
         convert_error::ConvertErrorWithContext,
         convert_error::ErrorContext,
-        error_reporter::convert_apis,
+        error_reporter::{convert_apis, report_any_error},
     },
     known_types::known_types,
     types::validate_ident_ok_for_rust,
@@ -40,8 +39,8 @@ use function_wrapper::{CppFunction, CppFunctionBody, TypeConversionPolicy};
 use itertools::Itertools;
 use proc_macro2::Span;
 use syn::{
-    parse_quote, punctuated::Punctuated, token::Comma, FnArg, ForeignItemFn, Ident, LitStr, Pat,
-    ReturnType, Type, TypePtr, Visibility,
+    parse_quote, punctuated::Punctuated, token::Comma, FnArg, Ident, Pat, ReturnType, Type,
+    TypePtr, Visibility,
 };
 
 use crate::{
@@ -56,12 +55,16 @@ use self::{
     bridge_name_tracker::BridgeNameTracker,
     overload_tracker::OverloadTracker,
     rust_name_tracker::RustNameTracker,
-    subclass::{add_subclass_constructor, create_subclass_function},
+    subclass::{
+        create_subclass_constructor, create_subclass_constructor_wrapper,
+        create_subclass_fn_wrapper, create_subclass_function,
+    },
 };
 
 use super::{
-    pod::{PodAnalysis, PodStructAnalysisBody},
-    tdef::TypedefAnalysisBody,
+    pod::{PodAnalysis, PodPhase},
+    tdef::TypedefAnalysis,
+    type_converter::Annotated,
 };
 
 #[derive(Clone, Debug)]
@@ -100,7 +103,7 @@ pub(crate) enum RustRenameStrategy {
 }
 
 #[derive(Clone)]
-pub(crate) struct FnAnalysisBody {
+pub(crate) struct FnAnalysis {
     pub(crate) cxxbridge_name: Ident,
     pub(crate) rust_name: String,
     pub(crate) rust_rename_strategy: RustRenameStrategy,
@@ -133,12 +136,12 @@ struct ReturnTypeAnalysis {
     deps: HashSet<QualifiedName>,
 }
 
-pub(crate) struct FnAnalysis;
+pub(crate) struct FnPhase;
 
-impl AnalysisPhase for FnAnalysis {
-    type TypedefAnalysis = TypedefAnalysisBody;
-    type StructAnalysis = PodStructAnalysisBody;
-    type FunAnalysis = FnAnalysisBody;
+impl AnalysisPhase for FnPhase {
+    type TypedefAnalysis = TypedefAnalysis;
+    type StructAnalysis = PodAnalysis;
+    type FunAnalysis = FnAnalysis;
 }
 
 pub(crate) struct FnAnalyzer<'a> {
@@ -151,14 +154,16 @@ pub(crate) struct FnAnalyzer<'a> {
     config: &'a IncludeCppConfig,
     overload_trackers_by_mod: HashMap<Namespace, OverloadTracker>,
     subclasses_by_superclass: HashMap<QualifiedName, Vec<SubclassName>>,
+    has_unrepresentable_constructors: HashSet<QualifiedName>,
+    nested_type_name_map: HashMap<QualifiedName, String>,
 }
 
 impl<'a> FnAnalyzer<'a> {
     pub(crate) fn analyze_functions(
-        apis: Vec<Api<PodAnalysis>>,
+        apis: Vec<Api<PodPhase>>,
         unsafe_policy: UnsafePolicy,
         config: &'a IncludeCppConfig,
-    ) -> Vec<Api<FnAnalysis>> {
+    ) -> Vec<Api<FnPhase>> {
         let mut me = Self {
             unsafe_policy,
             rust_name_tracker: RustNameTracker::new(),
@@ -169,6 +174,8 @@ impl<'a> FnAnalyzer<'a> {
             overload_trackers_by_mod: HashMap::new(),
             pod_safe_types: Self::build_pod_safe_type_set(&apis),
             subclasses_by_superclass: subclass::subclasses_by_superclass(&apis),
+            has_unrepresentable_constructors: HashSet::new(),
+            nested_type_name_map: Self::build_nested_type_map(&apis),
         };
         let mut results = Vec::new();
         convert_apis(
@@ -179,16 +186,17 @@ impl<'a> FnAnalyzer<'a> {
             Api::enum_unchanged,
             Api::typedef_unchanged,
         );
+        me.add_missing_make_uniques(&mut results);
         results.extend(me.extra_apis.into_iter().map(add_analysis));
         results
     }
 
-    fn build_pod_safe_type_set(apis: &[Api<PodAnalysis>]) -> HashSet<QualifiedName> {
+    fn build_pod_safe_type_set(apis: &[Api<PodPhase>]) -> HashSet<QualifiedName> {
         apis.iter()
             .filter_map(|api| match api {
                 Api::Struct {
                     analysis:
-                        PodStructAnalysisBody {
+                        PodAnalysis {
                             kind: TypeKind::Pod,
                             ..
                         },
@@ -213,25 +221,37 @@ impl<'a> FnAnalyzer<'a> {
             .collect()
     }
 
+    /// Builds a mapping from a qualified type name to the last 'nest'
+    /// of its name, if it has multiple elements.
+    fn build_nested_type_map(apis: &[Api<PodPhase>]) -> HashMap<QualifiedName, String> {
+        apis.iter()
+            .filter_map(|api| match api {
+                Api::Struct { name, .. } | Api::Enum { name, .. } => {
+                    let cpp_name = name
+                        .cpp_name
+                        .as_deref()
+                        .unwrap_or_else(|| name.name.get_final_item());
+                    cpp_name
+                        .rsplit_once("::")
+                        .map(|(_, suffix)| (name.name.clone(), suffix.to_string()))
+                }
+                _ => None,
+            })
+            .collect()
+    }
+
     fn convert_boxed_type(
         &mut self,
         ty: Box<Type>,
         ns: &Namespace,
         convert_ptrs_to_references: bool,
-    ) -> Result<(Box<Type>, HashSet<QualifiedName>, bool), ConvertError> {
-        let annotated = self.type_converter.convert_boxed_type(
-            ty,
-            ns,
-            &TypeConversionContext::CxxOuterType {
-                convert_ptrs_to_references,
-            },
-        )?;
-        self.extra_apis.extend(annotated.extra_apis);
-        Ok((
-            annotated.ty,
-            annotated.types_encountered,
-            annotated.requires_unsafe,
-        ))
+    ) -> Result<Annotated<Box<Type>>, ConvertError> {
+        let ctx = TypeConversionContext::CxxOuterType {
+            convert_ptrs_to_references,
+        };
+        let mut annotated = self.type_converter.convert_boxed_type(ty, ns, &ctx)?;
+        self.extra_apis.append(&mut annotated.extra_apis);
+        Ok(annotated)
     }
 
     fn get_cxx_bridge_name(
@@ -261,9 +281,9 @@ impl<'a> FnAnalyzer<'a> {
     fn analyze_foreign_fn_and_subclasses(
         &mut self,
         name: ApiName,
-        func_information: Box<FuncToConvert>,
-    ) -> Result<Box<dyn Iterator<Item = Api<FnAnalysis>>>, ConvertErrorWithContext> {
-        let maybe_analysis_and_name = self.analyze_foreign_fn(name, func_information.clone())?;
+        fun: Box<FuncToConvert>,
+    ) -> Result<Box<dyn Iterator<Item = Api<FnPhase>>>, ConvertErrorWithContext> {
+        let maybe_analysis_and_name = self.analyze_foreign_fn(name, fun.clone())?;
 
         let (analysis, name) = match maybe_analysis_and_name {
             None => return Ok(Box::new(std::iter::empty())),
@@ -276,13 +296,22 @@ impl<'a> FnAnalyzer<'a> {
         match &analysis.kind {
             FnKind::Method(sup, MethodKind::Constructor) => {
                 for sub in self.subclasses_by_superclass(sup) {
-                    if analysis.param_details.len() > 0 {
-                        return Err(ConvertErrorWithContext(
-                            ConvertError::SuperclassConstructorsTooFancy(sup.clone()),
-                            None,
-                        ));
+                    // Add a constructor to the actual subclass definition in pure C++
+                    results.push(create_subclass_constructor(&sub, &analysis, sup));
+                    // And consider adding an API (in Rust/cxx/maybe C++) such that we
+                    // can call this from Rust.
+                    let (maybe_wrap, subclass_constructor_name) =
+                        create_subclass_constructor_wrapper(sub, &fun);
+                    let maybe_another_api =
+                        self.analyze_foreign_fn(subclass_constructor_name, maybe_wrap)?;
+                    if let Some((analysis, name)) = maybe_another_api {
+                        results.push(Api::Function {
+                            fun: fun.clone(),
+                            analysis,
+                            name,
+                            name_for_gc: None,
+                        });
                     }
-                    add_subclass_constructor(&sub, &mut results, &analysis, &func_information);
                 }
             }
             FnKind::Method(
@@ -291,24 +320,29 @@ impl<'a> FnAnalyzer<'a> {
                 | MethodKind::PureVirtual(receiver_mutability),
             ) => {
                 for sub in self.subclasses_by_superclass(sup) {
-                    let subclass_function =
-                        create_subclass_function(&sub, &analysis, &name, receiver_mutability, sup);
-                    results.push(subclass_function);
+                    // For each subclass, we need to create a plain-C++ method to call its superclass
+                    // and a Rust/C++ bridge API to call _that_.
+                    // What we're generating here is entirely about the subclass, so the
+                    // superclass's namespace is irrelevant. We generate
+                    // all subclasses in the root namespace.
+                    let super_fn_name =
+                        SubclassName::get_super_fn_name(&Namespace::new(), &analysis.rust_name);
 
-                    let mut item = func_information.item.clone();
-                    item.sig.ident = SubclassName::get_super_fn_name(&analysis.rust_name);
-                    let self_ty = Some(QualifiedName::new(&Namespace::new(), sub.cpp()));
-                    let maybe_wrap = Box::new(FuncToConvert {
-                        item,
-                        virtual_this_type: self_ty.clone(),
-                        self_ty,
-                    });
-                    let name =
-                        ApiName::new(name.name.get_namespace(), maybe_wrap.item.sig.ident.clone());
-                    let maybe_another_api = self.analyze_foreign_fn(name, maybe_wrap)?;
+                    results.push(create_subclass_function(
+                        &sub,
+                        &analysis,
+                        &name,
+                        receiver_mutability,
+                        sup,
+                        &super_fn_name,
+                    ));
+
+                    let (maybe_wrap, super_fn_name) =
+                        create_subclass_fn_wrapper(sub, super_fn_name, &fun);
+                    let maybe_another_api = self.analyze_foreign_fn(super_fn_name, maybe_wrap)?;
                     if let Some((analysis, name)) = maybe_another_api {
                         results.push(Api::Function {
-                            fun: func_information.clone(),
+                            fun: fun.clone(),
                             analysis,
                             name,
                             name_for_gc: None,
@@ -320,7 +354,7 @@ impl<'a> FnAnalyzer<'a> {
         }
 
         results.push(Api::Function {
-            fun: func_information,
+            fun,
             analysis,
             name,
             name_for_gc: None,
@@ -347,10 +381,9 @@ impl<'a> FnAnalyzer<'a> {
     fn analyze_foreign_fn(
         &mut self,
         name: ApiName,
-        func_information: Box<FuncToConvert>,
-    ) -> Result<Option<(FnAnalysisBody, ApiName)>, ConvertErrorWithContext> {
-        let fun = &func_information.item;
-        let virtual_this = &func_information.virtual_this_type;
+        fun: Box<FuncToConvert>,
+    ) -> Result<Option<(FnAnalysis, ApiName)>, ConvertErrorWithContext> {
+        let virtual_this = &fun.virtual_this_type;
         let mut cpp_name = name.cpp_name.clone();
         let ns = name.name.get_namespace();
 
@@ -358,7 +391,7 @@ impl<'a> FnAnalyzer<'a> {
         // We're shortly going to plunge into analyzing the parameters,
         // and it would be nice to have some idea of the function name
         // for diagnostics whilst we do that.
-        let initial_rust_name = fun.sig.ident.to_string();
+        let initial_rust_name = fun.ident.to_string();
         if initial_rust_name.ends_with("_destructor") {
             return Ok(None);
         }
@@ -366,9 +399,7 @@ impl<'a> FnAnalyzer<'a> {
 
         // Now let's analyze all the parameters.
         // See if any have annotations which our fork of bindgen has craftily inserted...
-        let (reference_params, reference_return) = Self::get_reference_parameters_and_return(fun);
         let (param_details, bads): (Vec<_>, Vec<_>) = fun
-            .sig
             .inputs
             .iter()
             .map(|i| {
@@ -377,7 +408,7 @@ impl<'a> FnAnalyzer<'a> {
                     ns,
                     diagnostic_display_name,
                     virtual_this.clone(),
-                    &reference_params,
+                    &fun.reference_args,
                 )
             })
             .partition(Result::is_ok);
@@ -400,7 +431,6 @@ impl<'a> FnAnalyzer<'a> {
 
         // End of parameter processing.
         // Work out naming, part one.
-        // The Rust name... it's more complicated.
         // bindgen may have mangled the name either because it's invalid Rust
         // syntax (e.g. a keyword like 'async') or it's an overload.
         // If the former, we respect that mangling. If the latter, we don't,
@@ -431,7 +461,7 @@ impl<'a> FnAnalyzer<'a> {
             None => {
                 // Even if we can't find a 'self' parameter this could conceivably
                 // be a static method.
-                let self_ty = func_information.self_ty.clone();
+                let self_ty = fun.self_ty.clone();
                 (self_ty.is_some(), self_ty, None)
             }
             Some((self_ty, receiver_mutability)) => {
@@ -459,36 +489,41 @@ impl<'a> FnAnalyzer<'a> {
             // strip off the class name.
             let overload_tracker = self.overload_trackers_by_mod.entry(ns.clone()).or_default();
             let mut rust_name = overload_tracker.get_method_real_name(type_ident, ideal_rust_name);
-            let method_kind = if rust_name.starts_with(&type_ident) {
-                // It's a constructor. bindgen generates
-                // fn new(this: *Type, ...args)
-                // We want
-                // fn make_unique(...args) -> Type
-                // which later code will convert to
-                // fn make_unique(...args) -> UniquePtr<Type>
-                // If there are multiple constructors, bindgen generates
-                // new, new1, new2 etc. and we'll keep those suffixes.
-                let constructor_suffix = &rust_name[type_ident.len()..];
-                rust_name = format!("make_unique{}", constructor_suffix);
-                // Strip off the 'this' arg.
-                params = params.into_iter().skip(1).collect();
-                param_details.remove(0);
-                MethodKind::Constructor
-            } else if is_static_method {
-                MethodKind::Static
-            } else {
-                let receiver_mutability =
-                    receiver_mutability.expect("Failed to find receiver details");
-                if param_details.iter().any(|pd| pd.is_virtual) {
-                    if has_attr(&fun.attrs, "bindgen_pure_virtual") {
-                        MethodKind::PureVirtual(receiver_mutability)
-                    } else {
-                        MethodKind::Virtual(receiver_mutability)
-                    }
+            let nested_type_ident = self
+                .nested_type_name_map
+                .get(&self_ty)
+                .map(|s| s.as_str())
+                .unwrap_or_else(|| self_ty.get_final_item());
+            let method_kind =
+                if let Some(constructor_suffix) = rust_name.strip_prefix(nested_type_ident) {
+                    // It's a constructor. bindgen generates
+                    // fn Type(this: *mut Type, ...args)
+                    // We want
+                    // fn make_unique(...args) -> Type
+                    // which later code will convert to
+                    // fn make_unique(...args) -> UniquePtr<Type>
+                    // If there are multiple constructors, bindgen generates
+                    // new, new1, new2 etc. and we'll keep those suffixes.
+                    rust_name = format!("make_unique{}", constructor_suffix);
+                    // Strip off the 'this' arg.
+                    params = params.into_iter().skip(1).collect();
+                    param_details.remove(0);
+                    MethodKind::Constructor
+                } else if is_static_method {
+                    MethodKind::Static
                 } else {
-                    MethodKind::Normal(receiver_mutability)
-                }
-            };
+                    let receiver_mutability =
+                        receiver_mutability.expect("Failed to find receiver details");
+                    if param_details.iter().any(|pd| pd.is_virtual) {
+                        if fun.is_pure_virtual {
+                            MethodKind::PureVirtual(receiver_mutability)
+                        } else {
+                            MethodKind::Virtual(receiver_mutability)
+                        }
+                    } else {
+                        MethodKind::Normal(receiver_mutability)
+                    }
+                };
             let error_context = ErrorContext::Method {
                 self_ty: self_ty.get_final_ident(),
                 method: make_ident(&rust_name),
@@ -510,6 +545,16 @@ impl<'a> FnAnalyzer<'a> {
             )
         };
 
+        // Skip private methods; but if we've a private constructor, keep
+        // a note of it.
+        if fun.is_private {
+            if let FnKind::Method(self_ty, MethodKind::Constructor) = &kind {
+                self.has_unrepresentable_constructors
+                    .insert(self_ty.clone());
+            }
+            return Ok(None);
+        }
+
         // The name we use within the cxx::bridge mod may be different
         // from both the C++ name and the Rust name, because it's a flat
         // namespace so we might need to prepend some stuff to make it unique.
@@ -541,20 +586,21 @@ impl<'a> FnAnalyzer<'a> {
             }
         }
         // Second, reject any functions handling types which we flake out on.
-        if has_attr(&fun.attrs, "bindgen_unused_template_param_in_arg_or_return") {
+        if fun.unused_template_param {
             return Err(contextualize_error(ConvertError::UnusedTemplateParam));
         }
 
-        // Reject move constructors.
-        if Self::is_move_constructor(fun) {
-            return Err(contextualize_error(
-                ConvertError::MoveConstructorUnsupported,
-            ));
-        }
-
         match kind {
             FnKind::Method(_, MethodKind::Static) => {}
             FnKind::Method(ref self_ty, _) => {
+                // Reject move constructors.
+                if fun.is_move_constructor {
+                    self.has_unrepresentable_constructors
+                        .insert(self_ty.clone());
+                    return Err(contextualize_error(
+                        ConvertError::MoveConstructorUnsupported,
+                    ));
+                }
                 if !known_types().is_cxx_acceptable_receiver(self_ty) {
                     return Err(contextualize_error(ConvertError::UnsupportedReceiver));
                 }
@@ -578,7 +624,7 @@ impl<'a> FnAnalyzer<'a> {
                 deps: std::iter::once(self_ty).cloned().collect(),
             }
         } else {
-            self.convert_return_type(&fun.sig.output, ns, reference_return)
+            self.convert_return_type(&fun.output, ns, fun.return_type_is_reference)
                 .map_err(contextualize_error)?
         };
         let mut deps = params_deps;
@@ -692,7 +738,7 @@ impl<'a> FnAnalyzer<'a> {
             None
         };
 
-        let vis = func_information.item.vis.clone();
+        let vis = fun.vis.clone();
 
         // Naming, part two.
         // Work out our final naming strategy.
@@ -719,7 +765,7 @@ impl<'a> FnAnalyzer<'a> {
             }
         };
 
-        let analysis = FnAnalysisBody {
+        let analysis = FnAnalysis {
             cxxbridge_name,
             rust_name: rust_name.clone(),
             rust_rename_strategy,
@@ -814,10 +860,14 @@ impl<'a> FnAnalyzer<'a> {
                     }
                     _ => old_pat,
                 };
-                let (new_ty, deps, requires_unsafe) =
-                    self.convert_boxed_type(pt.ty, ns, treat_as_reference)?;
-                let was_reference = matches!(new_ty.as_ref(), Type::Reference(_));
-                let conversion = self.argument_conversion_details(&new_ty);
+                let annotated_type = self.convert_boxed_type(pt.ty, ns, treat_as_reference)?;
+                let new_ty = annotated_type.ty;
+                let subclass_holder = match &annotated_type.kind {
+                    type_converter::TypeKind::SubclassHolder(holder) => Some(holder),
+                    _ => None,
+                };
+                let conversion =
+                    self.argument_conversion_details(&new_ty, &subclass_holder.cloned());
                 pt.pat = Box::new(new_pat.clone());
                 pt.ty = new_ty;
                 (
@@ -826,10 +876,17 @@ impl<'a> FnAnalyzer<'a> {
                         self_type,
                         name: new_pat,
                         conversion,
-                        was_reference,
-                        deps,
+                        was_reference: matches!(
+                            annotated_type.kind,
+                            type_converter::TypeKind::Reference
+                                | type_converter::TypeKind::MutableReference
+                        ),
+                        deps: annotated_type.types_encountered,
                         is_virtual,
-                        requires_unsafe,
+                        requires_unsafe: matches!(
+                            annotated_type.kind,
+                            type_converter::TypeKind::Pointer
+                        ),
                     },
                 )
             }
@@ -837,7 +894,20 @@ impl<'a> FnAnalyzer<'a> {
         })
     }
 
-    fn argument_conversion_details(&self, ty: &Type) -> TypeConversionPolicy {
+    fn argument_conversion_details(
+        &self,
+        ty: &Type,
+        is_subclass_holder: &Option<Ident>,
+    ) -> TypeConversionPolicy {
+        if let Some(holder_id) = is_subclass_holder {
+            let subclass = SubclassName::from_holder_name(holder_id);
+            return TypeConversionPolicy::box_up_subclass_holder(
+                parse_quote! {
+                    rust::Box<#holder_id>
+                },
+                subclass,
+            );
+        }
         match ty {
             Type::Path(p) => {
                 let tn = QualifiedName::from_type_path(p);
@@ -884,60 +954,96 @@ impl<'a> FnAnalyzer<'a> {
             },
             ReturnType::Type(rarrow, boxed_type) => {
                 // TODO remove the below clone
-                let (boxed_type, deps, _) =
+                let annotated_type =
                     self.convert_boxed_type(boxed_type.clone(), ns, convert_ptr_to_reference)?;
+                let boxed_type = annotated_type.ty;
                 let was_reference = matches!(boxed_type.as_ref(), Type::Reference(_));
                 let conversion = self.return_type_conversion_details(boxed_type.as_ref());
                 ReturnTypeAnalysis {
                     rt: ReturnType::Type(*rarrow, boxed_type),
                     conversion: Some(conversion),
                     was_reference,
-                    deps,
+                    deps: annotated_type.types_encountered,
                 }
             }
         };
         Ok(result)
     }
 
-    fn get_bindgen_special_member_annotation(fun: &ForeignItemFn) -> Option<String> {
-        fun.attrs
-            .iter()
-            .filter_map(|a| {
-                if a.path.is_ident("bindgen_special_member") {
-                    let r: Result<LitStr, syn::Error> = a.parse_args();
-                    match r {
-                        Ok(ls) => Some(ls.value()),
-                        Err(_) => None,
-                    }
-                } else {
-                    None
-                }
-            })
-            .next()
-    }
-
-    fn is_move_constructor(fun: &ForeignItemFn) -> bool {
-        Self::get_bindgen_special_member_annotation(fun).map_or(false, |val| val == "move_ctor")
-    }
-
-    fn get_reference_parameters_and_return(fun: &ForeignItemFn) -> (HashSet<Ident>, bool) {
-        let mut ref_params = HashSet::new();
-        let mut ref_return = false;
-        for a in &fun.attrs {
-            if a.path.is_ident("bindgen_ret_type_reference") {
-                ref_return = true;
-            } else if a.path.is_ident("bindgen_arg_type_reference") {
-                let r: Result<Ident, syn::Error> = a.parse_args();
-                if let Ok(ls) = r {
-                    ref_params.insert(ls);
-                }
+    /// If a type has explicit constructors, bindgen will generate corresponding
+    /// constructor functions, which we'll have already converted to make_unique methods.
+    /// For types with no constructors, we synthesize one here.
+    /// It is tempting to make this a separate analysis phase, to be run later than
+    /// the function analysis; but that would make the code much more complex as it
+    /// would need to output a `FnAnalysisBody`. By running it as part of this phase
+    /// we can simply generate the sort of thing bindgen generates, then ask
+    /// the existing code in this phase to figure out what to do with it.
+    fn add_missing_make_uniques(&mut self, apis: &mut Vec<Api<FnPhase>>) {
+        if self.config.exclude_impls {
+            return;
+        }
+        let types_without_constructors = Self::find_all_types(apis);
+        // For types with private constructors, we won't have generated code for them,
+        // but we equally don't want to synthesize a public constructor. The same applies
+        // to other types of constructor we might skip e.g. move constructors.
+        let mut types_without_constructors =
+            &types_without_constructors - &self.has_unrepresentable_constructors;
+        // Now subtract all the actual constructors we know of.
+        for api in apis.iter() {
+            if let Api::Function {
+                analysis:
+                    FnAnalysis {
+                        kind: FnKind::Method(self_ty, MethodKind::Constructor),
+                        ..
+                    },
+                ..
+            } = api
+            {
+                types_without_constructors.remove(self_ty);
             }
         }
-        (ref_params, ref_return)
+        // We are left with those types where we should synthesize a constructor.
+        for self_ty in types_without_constructors {
+            let ident = self_ty.get_final_ident();
+            let fake_api_name = ApiName::new(self_ty.get_namespace(), ident.clone());
+            let ns = self_ty.get_namespace().clone();
+            let path = self_ty.to_type_path();
+            let items = report_any_error(&ns, apis, || {
+                self.analyze_foreign_fn_and_subclasses(
+                    fake_api_name,
+                    Box::new(FuncToConvert {
+                        virtual_this_type: Some(self_ty.clone()),
+                        self_ty: Some(self_ty),
+                        ident,
+                        doc_attr: None,
+                        inputs: parse_quote! { this: *mut #path },
+                        output: ReturnType::Default,
+                        vis: parse_quote! { pub },
+                        is_pure_virtual: false,
+                        is_private: false,
+                        is_move_constructor: false,
+                        unused_template_param: false,
+                        return_type_is_reference: false,
+                        reference_args: HashSet::new(),
+                        original_name: None,
+                    }),
+                )
+            });
+            apis.extend(items.into_iter().flatten());
+        }
+    }
+
+    fn find_all_types(apis: &[Api<FnPhase>]) -> HashSet<QualifiedName> {
+        apis.iter()
+            .filter_map(|api| match api {
+                Api::Struct { .. } => Some(api.name().clone()),
+                _ => None,
+            })
+            .collect::<HashSet<_>>()
     }
 }
 
-impl Api<FnAnalysis> {
+impl Api<FnPhase> {
     pub(crate) fn typename_for_allowlist(&self) -> QualifiedName {
         match &self {
             Api::Function {
@@ -991,7 +1097,7 @@ impl Api<FnAnalysis> {
         match self {
             Api::Typedef {
                 old_tyname,
-                analysis: TypedefAnalysisBody { deps, .. },
+                analysis: TypedefAnalysis { deps, .. },
                 ..
             } => Box::new(old_tyname.iter().chain(deps.iter())),
             Api::Struct { analysis, .. } => Box::new(analysis.field_deps.iter()),
@@ -1000,9 +1106,7 @@ impl Api<FnAnalysis> {
                 name: _,
                 superclass,
             } => Box::new(std::iter::once(superclass)),
-            Api::RustSubclassFn { details, .. } => {
-                Box::new(std::iter::once(&details.super_fn_api_name))
-            }
+            Api::RustSubclassFn { details, .. } => Box::new(std::iter::once(&details.dependency)),
             _ => Box::new(std::iter::empty()),
         }
     }
