@@ -25,11 +25,11 @@ use crate::{
     types::{validate_ident_ok_for_cxx, QualifiedName},
 };
 
-use super::fun::FnAnalysis;
+use super::fun::FnPhase;
 
 /// Do some final checks that the names we've come up with can be represented
 /// within cxx.
-pub(crate) fn check_names(apis: Vec<Api<FnAnalysis>>) -> Vec<Api<FnAnalysis>> {
+pub(crate) fn check_names(apis: Vec<Api<FnPhase>>) -> Vec<Api<FnPhase>> {
     // If any items have names which can't be represented by cxx,
     // abort. This check should ideally be done at the times we fill in the
     // `name` field of each `api` in the first place, at parse time, though
@@ -75,6 +75,7 @@ pub(crate) fn check_names(apis: Vec<Api<FnAnalysis>>) -> Vec<Api<FnAnalysis>> {
         | Api::RustType { .. }
         | Api::RustSubclassFn { .. }
         | Api::RustSubclassConstructor { .. }
+        | Api::RustFn { .. }
         | Api::IgnoredItem { .. } => Ok(Box::new(std::iter::once(api))),
     });
 
