@@ -14,7 +14,7 @@
 
 use std::collections::HashSet;
 
-use super::fun::{FnAnalysis, FnAnalysisBody, FnKind};
+use super::fun::{FnAnalysis, FnKind, FnPhase};
 use crate::conversion::api::ApiName;
 use crate::conversion::{convert_error::ErrorContext, ConvertError};
 use crate::{conversion::api::Api, known_types};
@@ -23,9 +23,7 @@ use crate::{conversion::api::Api, known_types};
 /// We also eliminate any APIs that depend on some type that we just don't
 /// know about at all. In either case, we don't simply remove the type, but instead
 /// replace it with an error marker.
-pub(crate) fn filter_apis_by_ignored_dependents(
-    mut apis: Vec<Api<FnAnalysis>>,
-) -> Vec<Api<FnAnalysis>> {
+pub(crate) fn filter_apis_by_ignored_dependents(mut apis: Vec<Api<FnPhase>>) -> Vec<Api<FnPhase>> {
     let (ignored_items, valid_items): (Vec<&Api<_>>, Vec<&Api<_>>) = apis.iter().partition(|api| {
         matches!(
             api,
@@ -41,8 +39,8 @@ pub(crate) fn filter_apis_by_ignored_dependents(
         .collect();
     let valid_types: HashSet<_> = valid_items
         .into_iter()
-        .map(|api| api.name())
-        .cloned()
+        .map(|api| api.valid_types())
+        .flatten()
         .collect();
     let mut iterate_again = true;
     while iterate_again {
@@ -54,15 +52,20 @@ pub(crate) fn filter_apis_by_ignored_dependents(
                     iterate_again = true;
                     ignored_items.insert(api.name().clone());
                     create_ignore_item(api, ConvertError::IgnoredDependent)
-                } else if !api
-                    .deps()
-                    .all(|dep| valid_types.contains(dep) || known_types().is_known_type(dep))
-                {
-                    iterate_again = true;
-                    ignored_items.insert(api.name().clone());
-                    create_ignore_item(api, ConvertError::UnknownDependentType)
                 } else {
-                    api
+                    let mut missing_deps = api
+                        .deps()
+                        .filter(|dep| {
+                            !valid_types.contains(dep) && !known_types().is_known_type(dep)
+                        })
+                        .cloned();
+                    let first = missing_deps.next();
+                    std::mem::drop(missing_deps);
+                    if let Some(missing_dep) = first {
+                        create_ignore_item(api, ConvertError::UnknownDependentType(missing_dep))
+                    } else {
+                        api
+                    }
                 }
             })
             .collect();
@@ -70,7 +73,7 @@ pub(crate) fn filter_apis_by_ignored_dependents(
     apis
 }
 
-fn create_ignore_item(api: Api<FnAnalysis>, err: ConvertError) -> Api<FnAnalysis> {
+fn create_ignore_item(api: Api<FnPhase>, err: ConvertError) -> Api<FnPhase> {
     let id = api.name().get_final_ident();
     log::info!("Marking as ignored: {} because {}", id.to_string(), err);
     Api::IgnoredItem {
@@ -82,7 +85,7 @@ fn create_ignore_item(api: Api<FnAnalysis>, err: ConvertError) -> Api<FnAnalysis
         ctx: match api {
             Api::Function {
                 analysis:
-                    FnAnalysisBody {
+                    FnAnalysis {
                         kind: FnKind::Method(self_ty, _),
                         ..
                     },
