@@ -22,6 +22,7 @@ use crate::{
     types::{make_ident, Namespace, QualifiedName},
 };
 use autocxx_parser::IncludeCppConfig;
+use proc_macro2::Ident;
 use quote::ToTokens;
 use std::collections::{HashMap, HashSet};
 use syn::{
@@ -29,7 +30,16 @@ use syn::{
     TypePath, TypePtr,
 };
 
-use super::tdef::TypedefAnalysisBody;
+use super::tdef::TypedefAnalysis;
+
+/// Certain kinds of type may require special handling by callers.
+pub(crate) enum TypeKind {
+    Regular,
+    Pointer,
+    SubclassHolder(Ident),
+    Reference,
+    MutableReference,
+}
 
 /// Results of some type conversion, annotated with a list of every type encountered,
 /// and optionally any extra APIs we need in order to use this type.
@@ -37,7 +47,7 @@ pub(crate) struct Annotated<T> {
     pub(crate) ty: T,
     pub(crate) types_encountered: HashSet<QualifiedName>,
     pub(crate) extra_apis: Vec<UnanalyzedApi>,
-    pub(crate) requires_unsafe: bool,
+    pub(crate) kind: TypeKind,
 }
 
 impl<T> Annotated<T> {
@@ -45,13 +55,13 @@ impl<T> Annotated<T> {
         ty: T,
         types_encountered: HashSet<QualifiedName>,
         extra_apis: Vec<UnanalyzedApi>,
-        requires_unsafe: bool,
+        kind: TypeKind,
     ) -> Self {
         Self {
             ty,
             types_encountered,
             extra_apis,
-            requires_unsafe,
+            kind,
         }
     }
 
@@ -60,7 +70,7 @@ impl<T> Annotated<T> {
             ty: fun(self.ty),
             types_encountered: self.types_encountered,
             extra_apis: self.extra_apis,
-            requires_unsafe: self.requires_unsafe,
+            kind: self.kind,
         }
     }
 }
@@ -158,7 +168,7 @@ impl<'a> TypeConverter<'a> {
                             }),
                             newp.types_encountered,
                             newp.extra_apis,
-                            false,
+                            TypeKind::Reference,
                         )
                     } else {
                         newp
@@ -175,7 +185,7 @@ impl<'a> TypeConverter<'a> {
                     Type::Reference(r),
                     innerty.types_encountered,
                     innerty.extra_apis,
-                    false,
+                    TypeKind::Reference,
                 )
             }
             Type::Ptr(ptr) if ctx.convert_ptrs_to_references() => {
@@ -190,7 +200,7 @@ impl<'a> TypeConverter<'a> {
                     Type::Ptr(ptr),
                     innerty.types_encountered,
                     innerty.extra_apis,
-                    true,
+                    TypeKind::Pointer,
                 )
             }
             _ => return Err(ConvertError::UnknownType(ty.to_token_stream().to_string())),
@@ -251,10 +261,17 @@ impl<'a> TypeConverter<'a> {
                     Type::Ptr(resolved_tp.clone()),
                     deps,
                     Vec::new(),
-                    true,
+                    TypeKind::Pointer,
+                ))
+            }
+            Some(other) => {
+                return Ok(Annotated::new(
+                    other.clone(),
+                    deps,
+                    Vec::new(),
+                    TypeKind::Regular,
                 ))
             }
-            Some(other) => return Ok(Annotated::new(other.clone(), deps, Vec::new(), false)),
         };
 
         // Now let's see if it's a known type.
@@ -273,6 +290,7 @@ impl<'a> TypeConverter<'a> {
         };
 
         let mut extra_apis = Vec::new();
+        let mut kind = TypeKind::Regular;
 
         // Finally let's see if it's generic.
         if let Some(last_seg) = Self::get_generic_args(&mut typ) {
@@ -282,7 +300,7 @@ impl<'a> TypeConverter<'a> {
             if generic_behavior != CxxGenericType::Not {
                 // this is a type of generic understood by cxx (e.g. CxxVector)
                 // so let's convert any generic type arguments. This recurses.
-                self.confirm_inner_type_is_acceptable_generic_payload(
+                kind = self.confirm_inner_type_is_acceptable_generic_payload(
                     &last_seg.arguments,
                     &tn,
                     generic_behavior,
@@ -304,7 +322,7 @@ impl<'a> TypeConverter<'a> {
                 deps.insert(new_tn);
             }
         }
-        Ok(Annotated::new(Type::Path(typ), deps, extra_apis, false))
+        Ok(Annotated::new(Type::Path(typ), deps, extra_apis, kind))
     }
 
     fn get_generic_args(typ: &mut TypePath) -> Option<&mut PathSegment> {
@@ -341,7 +359,7 @@ impl<'a> TypeConverter<'a> {
             new_pun,
             types_encountered,
             extra_apis,
-            false,
+            TypeKind::Regular,
         ))
     }
 
@@ -366,14 +384,20 @@ impl<'a> TypeConverter<'a> {
         // a wobbler if not. rust::Str should only be seen _by value_ in C++
         // headers; it manifests as &str in Rust but on the C++ side it must
         // be a plain value. We should detect and abort.
-        Ok(elem.map(|elem| match mutability {
+        let mut outer = elem.map(|elem| match mutability {
             Some(_) => Type::Path(parse_quote! {
                 ::std::pin::Pin < & #mutability #elem >
             }),
             None => Type::Reference(parse_quote! {
                 & #elem
             }),
-        }))
+        });
+        outer.kind = if mutability.is_some() {
+            TypeKind::MutableReference
+        } else {
+            TypeKind::Reference
+        };
+        Ok(outer)
     }
 
     fn get_templated_typename(
@@ -410,11 +434,11 @@ impl<'a> TypeConverter<'a> {
         desc: &QualifiedName,
         generic_behavior: CxxGenericType,
         forward_declarations_ok: bool,
-    ) -> Result<(), ConvertError> {
+    ) -> Result<TypeKind, ConvertError> {
         // For now, all supported generics accept the same payloads. This
         // may change in future in which case we'll need to accept more arguments here.
         match path_args {
-            PathArguments::None => Ok(()),
+            PathArguments::None => Ok(TypeKind::Regular),
             PathArguments::Parenthesized(_) => Err(
                 ConvertError::TemplatedTypeContainingNonPathArg(desc.clone()),
             ),
@@ -437,6 +461,16 @@ impl<'a> TypeConverter<'a> {
                                 if !self.config.is_rust_type(&inner_qn.get_final_ident()) {
                                     return Err(ConvertError::BoxContainingNonRustType(inner_qn));
                                 }
+                                if self
+                                    .config
+                                    .is_subclass_holder(&inner_qn.get_final_ident().to_string())
+                                {
+                                    return Ok(TypeKind::SubclassHolder(
+                                        inner_qn.get_final_ident(),
+                                    ));
+                                } else {
+                                    return Ok(TypeKind::Regular);
+                                }
                             }
                             if let Some(more_generics) = typ.path.segments.last() {
                                 self.confirm_inner_type_is_acceptable_generic_payload(
@@ -454,7 +488,7 @@ impl<'a> TypeConverter<'a> {
                         }
                     }
                 }
-                Ok(())
+                Ok(TypeKind::Regular)
             }
         }
     }
@@ -475,7 +509,8 @@ impl<'a> TypeConverter<'a> {
                 | Api::CType { .. }
                 | Api::RustSubclassFn { .. }
                 | Api::RustSubclassConstructor { .. }
-                | Api::IgnoredItem { .. } => None,
+                | Api::IgnoredItem { .. }
+                | Api::RustFn { .. } => None,
             })
             .cloned()
             .collect()
@@ -550,7 +585,7 @@ impl TypedefTarget for () {
     }
 }
 
-impl TypedefTarget for TypedefAnalysisBody {
+impl TypedefTarget for TypedefAnalysis {
     fn get_target(&self) -> Option<&Type> {
         match self.kind {
             TypedefKind::Type(ref ty) => Some(&ty.ty),
