@@ -12,10 +12,13 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::collections::HashSet;
+
 use crate::types::{make_ident, Namespace, QualifiedName};
+use autocxx_parser::RustPath;
 use syn::{
-    punctuated::Punctuated, token::Comma, FnArg, ForeignItemFn, Ident, ImplItem, ItemConst,
-    ItemEnum, ItemStruct, ItemType, ItemUse, ReturnType, Type,
+    punctuated::Punctuated, token::Comma, Attribute, FnArg, Ident, ImplItem, ItemConst, ItemEnum,
+    ItemStruct, ItemType, ItemUse, ReturnType, Signature, Type, Visibility,
 };
 
 use super::{
@@ -26,22 +29,17 @@ use super::{
 
 #[derive(Copy, Clone, Eq, PartialEq)]
 pub(crate) enum TypeKind {
-    Pod,    // trivial. Can be moved and copied in Rust.
-    NonPod, // has destructor or non-trivial move constructors. Can only hold by UniquePtr
+    Pod,          // trivial. Can be moved and copied in Rust.
+    NonPod,       // has destructor or non-trivial move constructors. Can only hold by UniquePtr
+    NonPodNested, // same, but nested inside another C++ struct/class.
+    // We have to do different codegen here because cxx can't cope with
+    // nested classes declared as 'type X;' so we instead have to do
+    // 'type X = super::bindgen::X;'
     Abstract, // has pure virtual members - can't even generate UniquePtr.
-            // It's possible that the type itself isn't pure virtual, but it inherits from
-            // some other type which is pure virtual. Alternatively, maybe we just don't
-            // know if the base class is pure virtual because it wasn't on the allowlist,
-            // in which case we'll err on the side of caution.
-}
-
-impl TypeKind {
-    pub(crate) fn can_be_instantiated(&self) -> bool {
-        match self {
-            TypeKind::Pod | TypeKind::NonPod => true,
-            TypeKind::Abstract => false,
-        }
-    }
+              // It's possible that the type itself isn't pure virtual, but it inherits from
+              // some other type which is pure virtual. Alternatively, maybe we just don't
+              // know if the base class is pure virtual because it wasn't on the allowlist,
+              // in which case we'll err on the side of caution.
 }
 
 /// An entry which needs to go into an `impl` block for a given type.
@@ -49,11 +47,25 @@ pub(crate) struct ImplBlockDetails {
     pub(crate) item: ImplItem,
     pub(crate) ty: Ident,
 }
-/// A ForeignItemFn with a little bit of context about the
-/// type which is most likely to be 'this'
+
+/// A C++ function for which we need to generate bindings, but haven't
+/// yet analyzed in depth. This is little more than a `ForeignItemFn`
+/// broken down into its constituent parts, plus some metadata from the
+/// surrounding bindgen parsing context.
 #[derive(Clone)]
 pub(crate) struct FuncToConvert {
-    pub(crate) item: ForeignItemFn,
+    pub(crate) ident: Ident,
+    pub(crate) doc_attr: Option<Attribute>,
+    pub(crate) inputs: Punctuated<FnArg, Comma>,
+    pub(crate) output: ReturnType,
+    pub(crate) vis: Visibility,
+    pub(crate) is_pure_virtual: bool,
+    pub(crate) is_private: bool,
+    pub(crate) is_move_constructor: bool,
+    pub(crate) unused_template_param: bool,
+    pub(crate) return_type_is_reference: bool,
+    pub(crate) reference_args: HashSet<Ident>,
+    pub(crate) original_name: Option<String>,
     pub(crate) virtual_this_type: Option<QualifiedName>,
     pub(crate) self_ty: Option<QualifiedName>,
 }
@@ -67,9 +79,9 @@ pub(crate) trait AnalysisPhase {
 }
 
 /// No analysis has been applied to this API.
-pub(crate) struct NullAnalysis;
+pub(crate) struct NullPhase;
 
-impl AnalysisPhase for NullAnalysis {
+impl AnalysisPhase for NullPhase {
     type TypedefAnalysis = ();
     type StructAnalysis = ();
     type FunAnalysis = ();
@@ -83,7 +95,7 @@ pub(crate) enum TypedefKind {
 
 /// Name information for an API. This includes the name by
 /// which we know it in Rust, and its C++ name, which may differ.
-#[derive(Clone, Hash, PartialEq, Eq, Debug)]
+#[derive(Clone, Hash, PartialEq, Eq)]
 pub(crate) struct ApiName {
     pub(crate) name: QualifiedName,
     pub(crate) cpp_name: Option<String>,
@@ -91,8 +103,19 @@ pub(crate) struct ApiName {
 
 impl ApiName {
     pub(crate) fn new(ns: &Namespace, id: Ident) -> Self {
+        Self::new_from_qualified_name(QualifiedName::new(ns, id))
+    }
+
+    pub(crate) fn new_with_cpp_name(ns: &Namespace, id: Ident, cpp_name: Option<String>) -> Self {
         Self {
             name: QualifiedName::new(ns, id),
+            cpp_name,
+        }
+    }
+
+    pub(crate) fn new_from_qualified_name(name: QualifiedName) -> Self {
+        Self {
+            name,
             cpp_name: None,
         }
     }
@@ -100,19 +123,39 @@ impl ApiName {
     pub(crate) fn new_in_root_namespace(id: Ident) -> Self {
         Self::new(&Namespace::new(), id)
     }
+
+    pub(crate) fn is_nested_struct_or_class(&self) -> bool {
+        self.cpp_name
+            .as_ref()
+            .map(|n| n.contains("::"))
+            .unwrap_or_default()
+    }
+}
+
+impl std::fmt::Debug for ApiName {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", self.name)?;
+        if let Some(cpp_name) = &self.cpp_name {
+            write!(f, " (cpp={})", cpp_name)?;
+        }
+        Ok(())
+    }
 }
 
 /// A name representing a subclass.
 /// This is a simple newtype wrapper which exists such that
 /// we can consistently generate the names of the various subsidiary
 /// types which are required both in C++ and Rust codegen.
-#[derive(Clone, Hash, PartialEq, Eq)]
+#[derive(Clone, Hash, PartialEq, Eq, Debug)]
 pub(crate) struct SubclassName(pub(crate) ApiName);
 
 impl SubclassName {
     pub(crate) fn new(id: Ident) -> Self {
         Self(ApiName::new_in_root_namespace(id))
     }
+    pub(crate) fn from_holder_name(id: &Ident) -> Self {
+        Self::new(make_ident(id.to_string().strip_suffix("Holder").unwrap()))
+    }
     pub(crate) fn id(&self) -> Ident {
         self.0.name.get_final_ident()
     }
@@ -121,8 +164,9 @@ impl SubclassName {
         self.with_suffix("Holder")
     }
     /// Generate the name for the 'Cpp' type
-    pub(crate) fn cpp(&self) -> Ident {
-        self.with_suffix("Cpp")
+    pub(crate) fn cpp(&self) -> QualifiedName {
+        let id = self.with_suffix("Cpp");
+        QualifiedName::new(self.0.name.get_namespace(), id)
     }
     pub(crate) fn cpp_remove_ownership(&self) -> Ident {
         self.with_suffix("Cpp_remove_ownership")
@@ -133,8 +177,20 @@ impl SubclassName {
     fn with_suffix(&self, suffix: &str) -> Ident {
         make_ident(format!("{}{}", self.0.name.get_final_item(), suffix))
     }
-    pub(crate) fn get_super_fn_name(id: &str) -> Ident {
-        make_ident(format!("{}_super", id))
+    pub(crate) fn get_super_fn_name(superclass_namespace: &Namespace, id: &str) -> QualifiedName {
+        let id = make_ident(format!("{}_super", id));
+        QualifiedName::new(superclass_namespace, id)
+    }
+    pub(crate) fn get_methods_trait_name(superclass_name: &QualifiedName) -> QualifiedName {
+        Self::with_qualified_name_suffix(superclass_name, "methods")
+    }
+    pub(crate) fn get_supers_trait_name(superclass_name: &QualifiedName) -> QualifiedName {
+        Self::with_qualified_name_suffix(superclass_name, "supers")
+    }
+
+    fn with_qualified_name_suffix(name: &QualifiedName, suffix: &str) -> QualifiedName {
+        let id = make_ident(format!("{}_{}", name.get_final_item(), suffix));
+        QualifiedName::new(name.get_namespace(), id)
     }
 }
 
@@ -212,7 +268,13 @@ pub(crate) enum Api<T: AnalysisPhase> {
         ctx: ErrorContext,
     },
     /// A Rust type which is not a C++ type.
-    RustType { name: ApiName },
+    RustType { name: ApiName, path: RustPath },
+    /// A function for the 'extern Rust' block which is not a C++ type.
+    RustFn {
+        name: ApiName,
+        sig: Signature,
+        path: RustPath,
+    },
     /// Some function for the extern "Rust" block.
     RustSubclassFn {
         name: ApiName,
@@ -224,6 +286,7 @@ pub(crate) enum Api<T: AnalysisPhase> {
         name: ApiName,
         subclass: SubclassName,
         cpp_impl: Box<CppFunction>,
+        is_trivial: bool,
     },
     /// A Rust subclass of a C++ class.
     Subclass {
@@ -239,7 +302,8 @@ pub(crate) struct RustSubclassFnDetails {
     pub(crate) method_name: Ident,
     pub(crate) superclass: QualifiedName,
     pub(crate) receiver_mutability: ReceiverMutability,
-    pub(crate) super_fn_api_name: QualifiedName,
+    pub(crate) dependency: QualifiedName,
+    pub(crate) requires_unsafe: bool,
 }
 
 impl<T: AnalysisPhase> Api<T> {
@@ -256,6 +320,7 @@ impl<T: AnalysisPhase> Api<T> {
             Api::CType { name, .. } => name,
             Api::IgnoredItem { name, .. } => name,
             Api::RustType { name, .. } => name,
+            Api::RustFn { name, .. } => name,
             Api::RustSubclassFn { name, .. } => name,
             Api::RustSubclassConstructor { name, .. } => name,
             Api::Subclass { name, .. } => &name.0,
@@ -285,15 +350,28 @@ impl<T: AnalysisPhase> Api<T> {
             .as_deref()
             .unwrap_or_else(|| self.name().get_final_item())
     }
+
+    pub(crate) fn valid_types(&self) -> Box<dyn Iterator<Item = QualifiedName>> {
+        match self {
+            Api::Subclass { name, .. } => Box::new(
+                vec![
+                    self.name().clone(),
+                    QualifiedName::new(&Namespace::new(), name.holder()),
+                ]
+                .into_iter(),
+            ),
+            _ => Box::new(std::iter::once(self.name().clone())),
+        }
+    }
 }
 
 impl<T: AnalysisPhase> std::fmt::Debug for Api<T> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{} (kind={})", self.name().to_cpp_name(), self)
+        write!(f, "{:?} (kind={})", self.name_info(), self)
     }
 }
 
-pub(crate) type UnanalyzedApi = Api<NullAnalysis>;
+pub(crate) type UnanalyzedApi = Api<NullPhase>;
 
 impl<T: AnalysisPhase> Api<T> {
     pub(crate) fn typedef_unchanged(
