@@ -16,6 +16,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+mod ast_discoverer;
 mod conversion;
 mod cxxbridge;
 mod known_types;
@@ -27,20 +28,12 @@ mod types;
 #[cfg(any(test, feature = "build"))]
 mod builder;
 
-#[cfg(test)]
-mod integration_tests;
-
 use autocxx_parser::{IncludeCppConfig, UnsafePolicy};
 use conversion::BridgeConverter;
 use parse_callbacks::AutocxxParseCallbacks;
 use parse_file::CppBuildable;
 use proc_macro2::TokenStream as TokenStream2;
-use std::{
-    collections::hash_map::DefaultHasher,
-    fmt::Display,
-    hash::{Hash, Hasher},
-    path::PathBuf,
-};
+use std::{fmt::Display, path::PathBuf};
 use std::{
     fs::File,
     io::prelude::*,
@@ -65,7 +58,9 @@ use log::info;
 use autocxx_bindgen as bindgen;
 
 #[cfg(any(test, feature = "build"))]
-pub use builder::{build, expect_build, BuilderBuild, BuilderError, BuilderResult, BuilderSuccess};
+pub use builder::{
+    Builder, BuilderBuild, BuilderContext, BuilderError, BuilderResult, BuilderSuccess,
+};
 pub use parse_file::{parse_file, ParseError, ParsedFile};
 
 pub use cxx_gen::HEADER;
@@ -83,7 +78,7 @@ pub struct CppFilePair {
     /// Declarations to go into a header file.
     pub header: Vec<u8>,
     /// Implementations to go into a .cpp file.
-    pub implementation: Vec<u8>,
+    pub implementation: Option<Vec<u8>>,
     /// The name which should be used for the header file
     /// (important as it may be `#include`d elsewhere)
     pub header_name: String,
@@ -112,7 +107,7 @@ impl Display for Error {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             Error::Bindgen(_) => write!(f, "Bindgen was unable to generate the initial .rs bindings for this file. This may indicate a parsing problem with the C++ headers.")?,
-            Error::Parsing(err) => write!(f, "The Rust file could not be parsede: {}", err)?,
+            Error::Parsing(err) => write!(f, "The Rust file could not be parsed: {}", err)?,
             Error::NoAutoCxxInc => write!(f, "No C++ include directory was provided.")?,
             Error::Conversion(err) => write!(f, "autocxx could not generate the requested bindings. {}", err)?,
         }
@@ -265,6 +260,14 @@ impl IncludeCppEngine {
         mac.parse_body::<IncludeCppEngine>().map_err(Error::Parsing)
     }
 
+    pub fn config_mut(&mut self) -> &mut IncludeCppConfig {
+        assert!(
+            matches!(self.state, State::NotGenerated),
+            "Can't alter config after generation commenced"
+        );
+        &mut self.config
+    }
+
     fn build_header(&self) -> String {
         join(
             self.config
@@ -318,10 +321,7 @@ impl IncludeCppEngine {
     }
 
     pub fn get_rs_filename(&self) -> String {
-        let mut hasher = DefaultHasher::new();
-        self.config.hash(&mut hasher);
-        let id = hasher.finish();
-        format!("{}.rs", id)
+        self.config.get_rs_filename()
     }
 
     /// Generate the Rust bindings. Call `generate` first.
@@ -361,6 +361,7 @@ impl IncludeCppEngine {
         inc_dirs: Vec<PathBuf>,
         extra_clang_args: &[&str],
         dep_recorder: Option<Box<dyn RebuildDependencyRecorder>>,
+        suppress_system_headers: bool,
     ) -> Result<()> {
         // If we are in parse only mode, do nothing. This is used for
         // doc tests to ensure the parsing is valid, but we can't expect
@@ -387,7 +388,12 @@ impl IncludeCppEngine {
         let converter = BridgeConverter::new(&self.config.inclusions, &self.config);
 
         let conversion = converter
-            .convert(bindings, self.config.unsafe_policy.clone(), header_contents)
+            .convert(
+                bindings,
+                self.config.unsafe_policy.clone(),
+                header_contents,
+                suppress_system_headers,
+            )
             .map_err(Error::Conversion)?;
         let mut items = conversion.rs;
         let mut new_bindings: ItemMod = parse_quote! {
@@ -475,26 +481,49 @@ static ALL_KNOWN_SYSTEM_HEADERS: &[&str] = &[
     "sys/types.h",
 ];
 
-pub fn do_cxx_cpp_generation(rs: TokenStream2) -> Result<CppFilePair, cxx_gen::Error> {
+pub fn do_cxx_cpp_generation(
+    rs: TokenStream2,
+    suppress_system_headers: bool,
+) -> Result<CppFilePair, cxx_gen::Error> {
     let opt = cxx_gen::Opt::default();
     let cxx_generated = cxx_gen::generate_header_and_cc(rs, &opt)?;
     Ok(CppFilePair {
-        header: cxx_generated.header,
+        header: strip_system_headers(cxx_generated.header, suppress_system_headers),
         header_name: "cxxgen.h".into(),
-        implementation: cxx_generated.implementation,
+        implementation: Some(strip_system_headers(
+            cxx_generated.implementation,
+            suppress_system_headers,
+        )),
     })
 }
 
+pub(crate) fn strip_system_headers(input: Vec<u8>, suppress_system_headers: bool) -> Vec<u8> {
+    if suppress_system_headers {
+        std::str::from_utf8(&input)
+            .unwrap()
+            .lines()
+            .filter(|l| !l.starts_with("#include <"))
+            .join("\n")
+            .as_bytes()
+            .to_vec()
+    } else {
+        input
+    }
+}
+
 impl CppBuildable for IncludeCppEngine {
     /// Generate C++-side bindings for these APIs. Call `generate` first.
-    fn generate_h_and_cxx(&self) -> Result<GeneratedCpp, cxx_gen::Error> {
+    fn generate_h_and_cxx(
+        &self,
+        suppress_system_headers: bool,
+    ) -> Result<GeneratedCpp, cxx_gen::Error> {
         let mut files = Vec::new();
         match &self.state {
             State::ParseOnly => panic!("Cannot generate C++ in parse-only mode"),
             State::NotGenerated => panic!("Call generate() first"),
             State::Generated(gen_results) => {
                 let rs = gen_results.item_mod.to_token_stream();
-                files.push(do_cxx_cpp_generation(rs)?);
+                files.push(do_cxx_cpp_generation(rs, suppress_system_headers)?);
                 if let Some(cpp_file_pair) = &gen_results.cpp {
                     files.push(cpp_file_pair.clone());
                 }
@@ -534,9 +563,7 @@ pub fn preprocess(
     cmd.arg(listing_path.to_str().unwrap());
     cmd.stderr(Stdio::inherit());
     let result = cmd.output().expect("failed to execute clang++");
-    if !result.status.success() {
-        panic!("failed to preprocess");
-    }
+    assert!(result.status.success(), "failed to preprocess");
     let mut file = File::create(preprocess_path)?;
     file.write_all(&result.stdout)?;
     Ok(())
