@@ -12,15 +12,18 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use crate::ast_discoverer::Discoveries;
 use crate::{
     cxxbridge::CxxBridge, Error as EngineError, GeneratedCpp, IncludeCppEngine,
     RebuildDependencyRecorder,
 };
-use proc_macro2::TokenStream;
+use autocxx_parser::directives::SUBCLASS;
+use autocxx_parser::{Subclass, SubclassAttrs};
+use proc_macro2::{Span, TokenStream};
 use quote::ToTokens;
 use std::{collections::HashSet, fmt::Display, io::Read, path::PathBuf};
 use std::{panic::UnwindSafe, path::Path, rc::Rc};
-use syn::Item;
+use syn::{Item, LitStr};
 
 /// Errors which may occur when parsing a Rust source file to discover
 /// and interpret include_cxx macros.
@@ -42,6 +45,9 @@ pub enum ParseError {
     /// There are two or more `include_cpp` macros with the same
     /// mod name.
     ConflictingModNames,
+    ZeroModsForDynamicDiscovery,
+    MultipleModsForDynamicDiscovery,
+    DiscoveredRustItemsWhenNotInAutoDiscover,
 }
 
 impl Display for ParseError {
@@ -54,24 +60,35 @@ impl Display for ParseError {
                 write!(f, "Unable to parse include_cpp! macro: {}", err)?,
             ParseError::ConflictingModNames =>
                 write!(f, "There are two or more include_cpp! macros with the same output mod name. Use name!")?,
+            ParseError::ZeroModsForDynamicDiscovery =>
+                write!(f, "This file contains extra information to append to an include_cpp! but no such include_cpp! was found in this file.")?,
+            ParseError::MultipleModsForDynamicDiscovery =>
+                write!(f, "This file contains extra information to append to an include_cpp! but multiple such include_cpp! declarations were found in this file.")?,
+            ParseError::DiscoveredRustItemsWhenNotInAutoDiscover =>
+                write!(f, "This file contains extra information to append to an \"extern Rust\" but auto-discover was switched off.")?,
         }
         Ok(())
     }
 }
 
 /// Parse a Rust file, and spot any include_cpp macros within it.
-pub fn parse_file<P1: AsRef<Path>>(rs_file: P1) -> Result<ParsedFile, ParseError> {
+pub fn parse_file<P1: AsRef<Path>>(
+    rs_file: P1,
+    auto_allowlist: bool,
+) -> Result<ParsedFile, ParseError> {
     let mut source = String::new();
     let mut file = std::fs::File::open(rs_file).map_err(ParseError::FileOpen)?;
     file.read_to_string(&mut source)
         .map_err(ParseError::FileRead)?;
     proc_macro2::fallback::force();
     let source = syn::parse_file(&source).map_err(ParseError::Syntax)?;
-    parse_file_contents(source)
+    parse_file_contents(source, auto_allowlist)
 }
 
-fn parse_file_contents(source: syn::File) -> Result<ParsedFile, ParseError> {
+fn parse_file_contents(source: syn::File, auto_allowlist: bool) -> Result<ParsedFile, ParseError> {
     let mut results = Vec::new();
+    let mut extra_superclasses = Vec::new();
+    let mut discoveries = Discoveries::default();
     for item in source.items {
         results.push(match item {
             Item::Macro(mac)
@@ -96,9 +113,88 @@ fn parse_file_contents(source: syn::File) -> Result<ParsedFile, ParseError> {
             {
                 Segment::Cxx(CxxBridge::from(itm))
             }
-            _ => Segment::Other(item),
+            Item::Struct(ref its) if auto_allowlist => {
+                let attrs = &its.attrs;
+                let is_superclass_attr = attrs.iter().find(|attr| {
+                    attr.path
+                        .segments
+                        .last()
+                        .map(|seg| seg.ident == "is_subclass" || seg.ident == SUBCLASS)
+                        .unwrap_or(false)
+                });
+                if let Some(is_superclass_attr) = is_superclass_attr {
+                    if !is_superclass_attr.tokens.is_empty() {
+                        let subclass = its.ident.clone();
+                        let args: SubclassAttrs = is_superclass_attr
+                            .parse_args()
+                            .map_err(ParseError::Syntax)?;
+                        if let Some(superclass) = args.superclass {
+                            extra_superclasses.push(Subclass {
+                                superclass,
+                                subclass,
+                            })
+                        }
+                    }
+                }
+                discoveries.search_item(&item);
+                Segment::Other(item)
+            }
+            _ => {
+                discoveries.search_item(&item);
+                Segment::Other(item)
+            }
         });
     }
+    if !auto_allowlist
+        && (!discoveries.extern_rust_types.is_empty() || !discoveries.extern_rust_funs.is_empty())
+    {
+        return Err(ParseError::DiscoveredRustItemsWhenNotInAutoDiscover);
+    }
+    if !extra_superclasses.is_empty() || (auto_allowlist && !discoveries.is_empty()) {
+        let mut autocxx_seg_iterator = results.iter_mut().filter_map(|seg| match seg {
+            Segment::Autocxx(engine) => Some(engine),
+            _ => None,
+        });
+        let our_seg = autocxx_seg_iterator.next();
+        match our_seg {
+            None => return Err(ParseError::ZeroModsForDynamicDiscovery),
+            Some(engine) => {
+                engine
+                    .config_mut()
+                    .subclasses
+                    .append(&mut extra_superclasses);
+                if auto_allowlist {
+                    for cpp in discoveries.cpp_list {
+                        engine
+                            .config_mut()
+                            .allowlist
+                            .push(LitStr::new(&cpp, Span::call_site()))
+                            .map_err(ParseError::Syntax)?;
+                    }
+                }
+                engine
+                    .config_mut()
+                    .extern_rust_funs
+                    .append(&mut discoveries.extern_rust_funs);
+                engine
+                    .config_mut()
+                    .rust_types
+                    .append(&mut discoveries.extern_rust_types);
+            }
+        }
+        if autocxx_seg_iterator.next().is_some() {
+            return Err(ParseError::MultipleModsForDynamicDiscovery);
+        }
+    }
+    let autocxx_seg_iterator = results.iter_mut().filter_map(|seg| match seg {
+        Segment::Autocxx(engine) => Some(engine),
+        _ => None,
+    });
+    for seg in autocxx_seg_iterator {
+        seg.config
+            .confirm_complete(auto_allowlist)
+            .map_err(ParseError::Syntax)?;
+    }
     Ok(ParsedFile(results))
 }
 
@@ -116,7 +212,10 @@ enum Segment {
 }
 
 pub trait CppBuildable {
-    fn generate_h_and_cxx(&self) -> Result<GeneratedCpp, cxx_gen::Error>;
+    fn generate_h_and_cxx(
+        &self,
+        suppress_system_headers: bool,
+    ) -> Result<GeneratedCpp, cxx_gen::Error>;
 }
 
 impl ParsedFile {
@@ -159,6 +258,7 @@ impl ParsedFile {
         autocxx_inc: Vec<PathBuf>,
         extra_clang_args: &[&str],
         dep_recorder: Option<Box<dyn RebuildDependencyRecorder>>,
+        suppress_system_headers: bool,
     ) -> Result<(), ParseError> {
         let mut mods_found = HashSet::new();
         let inner_dep_recorder: Option<Rc<dyn RebuildDependencyRecorder>> =
@@ -176,7 +276,12 @@ impl ParsedFile {
                 return Err(ParseError::ConflictingModNames);
             }
             include_cpp
-                .generate(autocxx_inc.clone(), extra_clang_args, dep_recorder)
+                .generate(
+                    autocxx_inc.clone(),
+                    extra_clang_args,
+                    dep_recorder,
+                    suppress_system_headers,
+                )
                 .map_err(ParseError::AutocxxCodegenError)?
         }
         Ok(())
